<?php
/* Copyright (C) 2010-2011 	Juanjo Menent		<jmenent@2byte.es>
 * Copyright (C) 2010		Laurent Destailleur	<eldy@users.sourceforge.net>
 * Copyright (C) 2011      	Regis Houssin		<regis.houssin@capnetworks.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * or see http://www.gnu.org/
 */

/**
 *	\file       htdocs/core/lib/lcr.lib.php
 *	\brief      Ensemble de fonctions de base pour le module lcr
 *	\ingroup    propal
 */


/**
 * Prepare array with list of tabs
 *
 * @param   Object	$object		Object related to tabs
 * @return  array				Array of tabs to shoc
 */
function lcr_prepare_head($object)
{
	global $langs, $conf, $user;
	$langs->load("withdrawals");

	$h = 0;
	$head = array();

	$head[$h][0] = dol_buildpath('/lcr/fiche.php?id='.$object->id, 1 );
	$head[$h][1] = $langs->trans("Card");
	$head[$h][2] = 'lcr';
	$h++;

// 	if (! empty($conf->global->MAIN_USE_PREVIEW_TABS))
// 	{
// 		$head[$h][0] = dol_buildpath('/lcr/tab/card.preview.php?id='.$object->id, 1 );
// 		$head[$h][1] = $langs->trans("Preview");
// 		$head[$h][2] = 'preview';
// 		$h++;
// // 	}

	$head[$h][0] = dol_buildpath('/lcr/tab/card.lignes.php?id='.$object->id, 1 );
	$head[$h][1] = $langs->trans("Lines");
	$head[$h][2] = 'lines';
	$h++;

	$head[$h][0] = dol_buildpath('/lcr/tab/card.factures.php?id='.$object->id, 1);
	$head[$h][1] = $langs->trans("Bills");
	$head[$h][2] = 'invoices';
	$h++;

	$head[$h][0] = dol_buildpath('/lcr/tab/card.fiche-rejet.php?id='.$object->id, 1 );
	$head[$h][1] = $langs->trans("Rejects");
	$head[$h][2] = 'rejects';
	$h++;

	$head[$h][0] = dol_buildpath('/lcr/tab/card.fiche-stat.php?id='.$object->id, 1 );
	$head[$h][1] = $langs->trans("Statistics");
	$head[$h][2] = 'statistics';
	$h++;

    // Show more tabs from modules
    // Entries must be declared in modules descriptor with line
    // $this->tabs = array('entity:+tabname:Title:@mymodule:/mymodule/mypage.php?id=__ID__');   to add new tab
    // $this->tabs = array('entity:-tabname);   												to remove a tab
    complete_head_from_modules($conf,$langs,$object,$head,$h,'lcr');

    complete_head_from_modules($conf,$langs,$object,$head,$h,'lcr','remove');

    return $head;
}

/**
 *	Check need data to create standigns orders receipt file
 *
 *	@return    	int		-1 if ko 0 if ok
 */
function lcr_check_config()
{
	global $conf;
  if(empty($conf->global->LCR_USER)) return -1;
	if(empty($conf->global->LCR_ID_BANKACCOUNT)) return -1;

	return 0;
}


/**
	@brief Create display header segment card of lcr
*/
function PrintHeadCardTab($card, $object, $prev_id){
	global $db, $langs, $user ; 

	
	if ($object->fetch($prev_id) == 0)
	{
		$head = lcr_prepare_head($object);
		
	dol_fiche_head($head, $card, $langs->trans("WithdrawalReceipts"), '', 'payment');

	$linkback = '<a href="' .dol_buildpath('/lcr/liste.php',1) . '?restore_lastsearch_values=1' . (! empty($socid) ? '&socid=' . $socid : '') . '">' . $langs->trans("BackToList") . '</a>';
	
	$morehtmlref='<div class="refidno">';
	$morehtmlref.= '</div>';
	
	$morehtmlstatus= $object->getLibStatut(1);

	dol_banner_tab($object, 'ref', $linkback, 1, 'ref', 'ref', $morehtmlref, '',0,'',$morehtmlstatus);

	print '<div class="fichecenter">'; 
	print '<div class="fichehalfleft">';
	print '<div class="underbanner clearboth"></div>';
	
		print '<table class="border tableforfield" width="100%">';

		print '<tr ><td class="titlefield" >'.$langs->trans("Date").'</td><td>'.dol_print_date($object->datec,'day').'</td></tr>';
		print '<tr><td class="titlefield" >'.$langs->trans("Amount").'</td><td>'.price($object->amount).'</td></tr>';

		print '<tr><td width="20%">';
		print $langs->trans("WithdrawalFile").'</td><td>';
		$relativepath = 'receipts/'.$object->ref."";
		print '[<a data-ajax="false" href="'.DOL_URL_ROOT.'/document.php?type=text/plain&amp;modulepart=lcr&amp;file='.urlencode($relativepath.".txt").'">'.$relativepath.'</a>] ';
		print '</td></tr>';

		
		print '</table>';
	print '</div>';
	
	
	print '<div class="fichehalfright">';
	print '<div class="underbanner clearboth"></div>';
	
		print '<table class="border tableforfield" width="100%">';

		if($object->date_trans <> 0)
		{
			
			$user->fetch($object->user_trans);

			print '<tr><td width="20%">'.$langs->trans("TransData").'</td><td>';
			print dol_print_date($object->date_trans,'day');
			print ' '.$langs->trans("By").' '.$user->getFullName($langs).'</td></tr>';
			print '<tr><td width="20%">'.$langs->trans("TransMetod").'</td><td>';
			print $object->methodes_trans[$object->method_trans];
			print '</td></tr>';
		}
		if($object->date_credit <> 0)
		{
			print '<tr><td width="20%">'.$langs->trans('CreditDate').'</td><td>';
			print dol_print_date($object->date_credit,'day');
			print '</td></tr>';
		}

		print '</table>';

	print '</div>';

	print '</div>';
	
	print '<div class="clearboth"></div><br>';
	
	dol_fiche_end();
	
			
	}
	else
	{
	
		$langs->load("errors");
		print $langs->trans("Error");
	}
    
}

